#ifndef BUNDLE_H
#define BUNDLE_H

#include <QtCore>

/*! \brief This class represents a bundle within PRoPHET.
 */
class Bundle : public QObject
{

Q_OBJECT 
	public:
	int id;
	int sourceId;
	int destinationId;
	QByteArray data;
	QByteArray sourceString;
	QByteArray destinationString;
	QDateTime timeStamp;
	int options;
	int dataLength;
	// The id of the last node that sent the bundle.
	// Used for keeping track of which bundles have been sent to which nodes.
	int lastSender;


	Bundle(QObject *parent = 0);
	Bundle(const Bundle& other);
	Bundle& operator=(const Bundle& other);

	/*! \brief Sets the content of this Bundle.
	 * @param seq The sequence number of this Bundle??? Used for 
	 * calculating the id.
	 * @param s The id of the source node this Bundle originated from.
	 * @param d The id of the destination node of this Bundle.
	 * @param m The data payload of this Bundle.
	 * @param opt The options for this bundle. Only the three LSBits are used
	 * \verbatim
	           3                2                   1              0
	  -------------------------------------------------------------------
	  ... | 1=Ack     | 0 = PRoPHET bundle | 1 = Ack me       | Not used |
	      | 0=Non-ack | 1 = DTN bundle     | 0 = Don't ack me |          |
	  -------------------------------------------------------------------
	  \endverbatim
	 * @param src The string name of the source node.
	 * @param dst The string name of the destination node.
	 */void setContent(int,int s,int d,QByteArray m,int opt,QByteArray src,QByteArray dst);
	
	/*!
	 * Sets the content of this Bundle.
	 * @param seq The sequence number of this Bundle??? Used for 
	 * calculating the id.
	 * @param s The id of the source node this Bundle originated from.
	 * @param d The id of the destination node of this Bundle.
	 * @param m The data payload of this Bundle.
	 * @param opt The options for this bundle. Only the three LSBits are used
	 * \verbatim
	          3                2                   1              0
	  -------------------------------------------------------------------
	  ... | 1=Ack     | 0 = PRoPHET bundle | 1 = Ack me       | Not used |
	      | 0=Non-ack | 1 = DTN bundle     | 0 = Don't ack me |          |
	  -------------------------------------------------------------------
	  \endverbatim
	 */
	void setContent(int,int,int,QByteArray,int);
	
	//! Sets the (non-PRoPHET) ack bit of this bundle.
	/*!
	 * This is used for ackin bundles on the connection level, so that each 
	 * bundle is only sent once in bundle offers.
	 */
	void setAck();
	
	//! Resets the (non-PRoPHET) ack bit of this bundle.
	/*!
	 * This is used for ackin bundles on the connection level, so that each 
	 * bundle is only sent once in bundle offers.
	 */
	void unSetAck();

	//! Checks whether this bundle has the (non PRoPHET) ack bit set.
	/*!
	 * @return true if the (non PRoPHET) ack bit is set, false otherwise.
	 */
	bool isAck();

	//! Sets the last sender of this bundle.
	/*!
	 * Used for keeping track of which bundles have already been sent to 
	 * certains nodes.
	 * @param sender the id of the last sender.
	 */
	void setLastSender(int sender){
		lastSender = sender;
	}

	//! Returns the id if the last sender
	/*!
	 * Used for keeping track of which bundles have already been sent to 
	 * certain nodes.
	 * @return the id of the last sender
	 */
	int getLastSender(){
		return lastSender;
	}

	/*!
	 * Returns the id of the Bundle.
	 * @return The id of this Bundle.
	 */
	int getId(); 
	
	/*!
	 * Returns the source id of this Bundle.
	 * @return The source id of this Bundle.
	 */
	int getSourceId();
	
	/*!
	 * Returns the destination id of this Bundle.
	 * @return The destination id of this Bundle.
	 */
	int getDestinationId();
	
	/*!
	 * Returns the timestamp of when this Bundle was created.
	 * @return The timestamp of when this Bundle was created.
	 */
	QDateTime getTime();
	
	/*!
	 * Returns the data payload of this Bundle.
	 * @return The data payload of this Bundle.
	 */
	QByteArray getData();
	
	/*!
	 * Returns the options of this Bundle. Only the three LSBits are used.
	 * \verbatim
	          3                2                   1              0
	  -------------------------------------------------------------------
	  ... | 1=Ack     | 0 = PRoPHET bundle | 1 = Ack me       | Not used |
	      | 0=Non-ack | 1 = DTN bundle     | 0 = Don't ack me |          |
	  -------------------------------------------------------------------
	  \endverbatim
	 */
	int getOptions();
	
};

#endif
