#ifndef BUNDLEMANAGER_H
#define BUNDLEMANAGER_H
#include <QtCore>
#include <bundle.h>
#include <node.h>
#include <nodeManager.h>
#include <messageFile.h>
#include <bundleListFileIO.h>

/*! \brief This class keeps track of bundles and handles bundle storage, both 
 * in memory and on disk. 
 *
 * PRoPHET routing is also done in this class (but should be moved 
 * to its own object.
 */
class BundleManager : public QObject
{
	Q_OBJECT

public:
	BundleManager(int);
	int getMemoryStorageSize();
	int getMemoryBundleSize(Bundle);
	int getFileStorageSize();
	int getFileBundleSize(Bundle);
	QList<Bundle> getBundleList();

	//Returns the list of bundles that are to be offered to the encountered node.
	/*!
	 * Returns the list of bundles that are to be offered to the encountered 
	 * node.
	 * @param ourList our list of known nodes (from node manager).
	 * @param encoutnerlist the node list of the encountered node.
	 * @param encounterId the node id of the encountered node.
	 * @return the list of bundles that are to be offered to the encountered 
	 * node.
	 */
	QList<Bundle> getBundleOffer(QList<Node> ourList,QList<Node> encoutnerList,int encounterId);
	QList<Bundle> prepareBundelRequest(QList<Bundle>);

	// Builds the list of bundles that are to be transferred to the encountered node.
	/*!
	 * @param receivedList the list of requested bundles.
	 * @param senderId the id of the node this Bundle was received from. 
	 * Note: This is NOT the source id.
	 * @return the list of bundles that will be sent to the other node.
	 */
	QList<Bundle> prepareBundels(QList<Bundle>, int senderId);

private:
	QTimer *timer;
	QTimer *TTLTimer;
	int routing;	//0=epidemic, 1=GRTR
	int fileOption;
	int TTLOption;
	int TTLValue;
	int ACKOption;
	int memoryStorageSize;
	int fileStorageSize;
	int ourId;
	int seq;
	int smartOffer;

	// keeps track of which nodes each bundle was sent to
	// key = bundle id, value = list of node ids
	// (should be declared private)
	QMultiHash<int, int> transferredBundleHash;
    BundleListFileIO* hashFile;

	QTimer *writeToFileTimer;
	QList<Bundle> bundleList;

	QDir dir;
	QDir msgDir;
	QString storagePath;
	int option;
	int logOption;
	MessageFile *log;
	bool showFirstTime;

signals:
	/*!
	 * Emits the specified Bundle. (Used for passing bundles to DTN)
	 */
	void sendDeliveredBundle(Bundle);
	
		/*!
	 * Emits the memory usage of the bundle list in bytes(?).
	 */
	void sendMemoryStorage(int);
	
	/*!
	 * Emits the size (in bytes?) of the file storage.
	 */
	void sendFileStorage(int);
	
	/*!
	 * Emits the bundle list (to the GUI).
	 */
	void sendDebugBundleList(QList<Bundle>);

public slots:
	/*!
	 * External method to get a  bundle sequence number.
	 */
	int getSeq();

	/*!
	 * External slot for receiving bundles. Received bundles are 
	 * appropriately dispatched to either the application layer (an NSIM 
	 * bundle) or to DTN.
	 */
	void addBundle(Bundle);
	
	/*!
	 * Goes through the bundle list searching for bundles who's destination 
	 * id=0 and assigning the correct destination for any such bundles. 
	 * This function is a workaround for the situation when bundles are 
	 * received before this client is aware of (has a probability for) the 
	 * destination node.
	 */
	void updateBundlesDestinations(NodeManager*);
	
	/*!
	 * Inserts all bundles in the list to the list of this bundleManager.
	 * @param receivedList a list of bundles to be added to this 
	 * bundleManagers list.
	 */
	void addBundleList(QList<Bundle>);
	
private slots:
	/*!
	 * Saves the bundle list to the file bundles.txt.
	 */
	void saveToFile();
	
	/*!
	 * Updates the bundle list in memory to match the one stored on disk in 
	 * bundles.txt.
	 */
	void readFromFile();
	
	/*!
	 * Removes the specified bundle from the list.
	 * @param bundleID the id of the bundle to be removed.
	 */
	bool removeBundle(int);
	
	/*!
	 * Triggers sending the bundle list to the GUI.
	 */	
	void emitDebugList();
	
	/*!
	 * Iterates through the bundle list and removes any bundles which have 
	 * an outdated timestamp.
	 */
	void checkTTL();
	
	/*!
	 * Removes all bundles from the list.
	 */
	void removeAllBundles();
	
	/*!
	 * Internal slot for adding a bundle to the list.
	 */
	void storeBundle(Bundle);


};
#endif
