#ifndef CONNECTION_H
#define CONNECTION_H
#include <QtCore>
#include <hello.h>
#include <tlv.h>
#include <nodeManager.h>
#include <bundleManager.h>
#include <dataPacket.h>


/*! \brief This class is instantiated for each connection between two nodes. 
 * It contains the PRoPHET state machine.
 *
 * <PRE>
 * Connection states
 * WAIT_NB   - Start state, node waiting for a neighbour to connect to.
 * HELLO     - Performs a three-way handshake to set up the connection. Also 
 *             assigns a MASTER and a SLAVE for the communication.
 * INFO_EXCH - Entered once the Hello procedure has set up the link. In this 
 *             state routing information and bundles are exchanged.
 * WAIT_INFO - Entered once the INFO_EXCH is completed. Nodes remain in this 
 *             state until a timer expires or they are notified that the 
 *             neighbor is out of range. WAIT_NB is then entered.
 *
 * Hello procedure states
 * SENDSYN     - Initiation of the Hello procedure. First part of the 
 *                three-way handshake.
 * WAITSYNACK  - Second part of the three-way handshake.
 * WAITACK     - Last part of the three-way handshake.
 * ESTABLISHED
 * MASTER      - One node is assigned to be MASTER, this node will enter the 
 *                initiator states.
 * SLAVE       - The other node will be assigned SLAVE, and will enter the 
 *                listener states.
 * 
 * Initiator states (MASTER)
 * SENDDICTIONARY
 * SENDRIB
 * SENDBOFFER
 * WAITBREQ
 * TIMEOUT
 * SENDBUNDLE
 * WAITFORBREQUEST
 * RESETLINK
 * IDLESTATE
 * 
 * Listener states (SLAVE)
 * WAITFORDICTIONARY
 * WAITFORRIB
 * WAITFORBOFFER
 * SENDBREQUEST
 * WAITFORBUNDLES
 * </PRE>
 */
class Connection: public QObject
{
	Q_OBJECT 
	public:
		Connection(int,int ,QString ,NodeManager* ,BundleManager* ,QObject *parent = 0);
		~Connection(void);

	private:
		int HELLO_TIMER;
		int INITIATOR_TIMER;
		int LISTENER_TIMER;
		int HELLOT;
		int BROADCAST;
		bool update;
		bool updateRIB;
		int continiusUpdate;

		int connectionState;
		int helloState;
		int initiatorState;
		int listenerState;
		int encounterNodeId;
		int encounterNodeType;
		QString encounterNodeName;
		QTimer *helloTimer; 		
		QTimer *initiatorTimer; 		
		QTimer *listenerTimer; 		
		//TLV
		TLV *TLVTranslator; 
		QString connectionNodeName;
		QList<Node> lastReceivedRIB;
		QList<Bundle> bundleRequestList;
		int connectionId;
		int connectionNodeId;
		NodeManager* connectionNodeManager;
		BundleManager* connectionBundleManager;

		

	signals:

		//! Sends a datapacket to neighAware.
		/*! Connected to NeighAware::sendDatagram
		 * @param packet the DataPacket to send.
		 */
		void sendDataPacket(DataPacket);
		
		//! Sends a Hello to the TLV.
		/*! Connected to TLV::receiveHello
		 * @param hello the Hello to be sent.
		 */
		void sendHello(Hello);

		//! Sends a dictionary to tht TLV.
		/*! Connected to TLV::createDictionary
		 * @param nodes the list of nodes to be included in the dictionary.
		 */
		void sendDictionary(QList<Node>);

		//! Sends a RIB to the TLV.
		/*! Connected to TLV::createRIB
		 * @param nodes the list of nodes to be included in the RIB.
		 */
		void sendRIB(QList<Node>);

		//! Sends a datagram to the TLV.
		/*! Connected to TLV::receiveDatagram
		 * @param datagram the datagram to be sent.
		 */
		void forwardSignalDatagram(QByteArray);

		//! Sends a bundle offer to the TLV.
		/*! Connected to TLV::createBundleOffer
		 * @param the list of bundles to be included in the offer.
		 */
		void sendBundleOffer(QList<Bundle>);

		//! Sends a bundle request to the TLV.
		/*! Connected to TLV::createBundleRequest.
		 * @param the list of bundles to be requested. (?)
		 */
		void sendBundleRequest(QList<Bundle>);

		//! Sends a list of bundles to the TLV.
		/*! Connected to TLV::createBundleData.
		 * @param the bundles to be sent.
		 */
		void sendBundles(QList<Bundle>);

		//! Sends the status of this connection.
		/*! Connected to NeighAware::getNewConnectionStatus. (Is this only 
		 * used for GUI???)
		 * @param the connection state(?).
		 * <PRE>
		 *	WAIT_NB   = 0
		 *	HELLO     = 1
		 *	INFO_EXCH = 2
		 *	WAIT_INFO = 2
		 * </PRE>
		 */
		void newConnectionStatus(int);
	public slots:

		//! Internal slot for receiving packets.
		/*! Extracts the data from the datagram and emits forwardSignalDatagram.
		 * @param dataPacket the datapacket received from the network. 
		 */
		void receiveDatagram(DataPacket);

	private slots:
		//! Internal(?) slot for sending packets.
		/*! Inserts the data into a DataPacket and emits sendDataPacket.
		 * @param datagram the data to be sent.
		 */
		void forwardSlotDatagram(QByteArray);

		//! Called every time the hello timer expires, initiates a new hello procedure.
		void helloProcedure();

		//! Called when the Hello procedure is completed if this node was given the SLAVE role.
		void listenerProcedure();

		//! Called when the Hello procedure is completed if this node was given the MASTER role.
		void initiatorProcedure();

		//! Slot for reception of hello messages.  
		/*! Checks the type of the hello and reacts accordingly by changing 
		 * state, and possibly sending a response.
		 */
		void receiveHello(Hello);

		//! Slot for handling the reception of a dictionary.
		/*! When a dictionary (node id, name mappings) is received it is 
		 * dispatched to the node manager.
		 * @param receivedList the list of (node id, name mappings)
		 */
		void receiveDictionary(QList<Node>);

		//! Slot for handling the reception of the RIB.
		/*! The RIB (Routing Information Base) contains node id -> delivery 
		 * predictability mappings.
		 * @param receivedList the RIB table.
		 */
		void receiveRIB(QList<Node>);

		//! Handles reception of bundle requests.
		/*! Prepares a bundle request based on the offer, and sends out a 
		 * request for the first bundle in the list. If no more bundles are to 
		 * be sent the connection is reset to the hello state and an empty 
		 * bundle request is sent to the other node to indicate this.
		 * @param receivedList the list of bundles that were offered by the 
		 * other node.
		 */
		void receiveBundleOffer(QList<Bundle>);

		//! This function is where the initiator handles bundle requests.
		/*! When a request for a list of bundles is received the appropriate 
		 * bundles are sent to the other node.
		 * @param receivedList the list of bundles the other node requested.
		 */
		void receiveBundleRequest(QList<Bundle>);

		//! This function is where the listener handles reception of bundles.
		/*! When a list of bundles is received (currently only one bundle at a 
		 * time) it is dispatched to the bundle manager and a new request is 
		 * sent out. If the bundle request list is empty an empty bundle 
		 * request is sent out to indicate this to the other node.
		 * @param receivedList the list of bundles received.
		 */
		void receiveBundles(QList<Bundle>);


};
#endif
